/*****************************************************************************/
/*                                                                           */
/*   LCD Back Light Control with AVR                                         */
/*                                                                           */
/*   (c) 2008 Dipl.-Ing. (FH) Rainer Reusch, Reusch Elektronik for Elektor   */
/*   http://www.reusch-elektronik.de                                         */
/*   http://reweb.fh-weingarten.de/elektor                                   */
/*                                                                           */
/*   Atmel AVR Studio V4.13.528                                              */
/*   GNU C-Compiler GCC V4.2.20                                              */
/*                                                                           */
/*   Created: 2008-02-19                                                     */
/*   Changed: 2008-02-21                                                     */
/*                                                                           */
/*****************************************************************************/

/*
Instructions
------------

Usage with feedback:
Call the routine Init_BacklightFB one time at startup.
Initial current value is given as parameter.
To change the current, simply change the contents 
of the global variable "Current".
Important: Interrupts has to be enabled!

Usage without feedback:
Call the routine Init_Backlight one time at startup.
Initial pwm value is given as parameter.
Call the routine SetPWM to change the pwm.

Preferences (developed and tested)
----------------------------------
CPU: ATmega32
Clock frequency: 8.0MHz -> pwm frequency 31.25kHz 
PWM output (backlight current): OC2 (PD7)
Analogue input (current feedback): ADC0 (PA0)

Module may work on other ATmega MCUs with no or tiny modifications.
For ATtiny MCUs the reduced peripherals has to be considered.
*/

//-----------------------------------------------------------------------------
// Port assignments
//-----------------------------------------------------------------------------
#define FBDir			DDRA		// data direction register of used feedback analogue input
#define FBPort		PORTA		// port register of used feedback analogue input
#define FBPin			PA0			// pin of used feedback analogue input
#define PWMDir		DDRD		// data direction register of used PWM port
#define PWMPort 	PORTD		// port register of used PWM port
#define PWMPin		PD7			// pin of used PWM port (OC2)

//-----------------------------------------------------------------------------
// Other definitions
//-----------------------------------------------------------------------------
#define NumAvrgPwr	4			// number of ADC values for average calculation (power of 2)
#define NumAvrgVals (1<<NumAvrgPwr)	// number of ADC values for average calculation (only values power of 2 !!!)

//-----------------------------------------------------------------------------
// Global variables
//-----------------------------------------------------------------------------
uint8_t Current;       		// input variable: backlight current (feedback mode)
uint8_t	PWMValue;      		// actual PWM Value, readonly, use SetPWM to change pwm value

//-----------------------------------------------------------------------------
// Routines
//-----------------------------------------------------------------------------
// with feedback
void Init_BacklightFB(uint8_t TurnOnCurrent);
// without feedback
void Init_Backlight(uint8_t TurnOnPWM);
void SetPWM(uint8_t pwm);
